# src/orphanet_explorer/__init__.py
from .orphanet_explorer import OrphanetDataManager
from .exceptions import OrphanetProcessorError

__version__ = "0.1.0"
__all__ = ["OrphanetDataManager", "OrphanetProcessorError"]
